/*************************************************************************
/ ENLACE.C
/ Implementa el nivel de enlace: interfaz entre el nivel de red y la
/ tarjeta Ethernet.
/*************************************************************************/

#include "enlace.h"

extern unsigned long CUENTA;

// Buffer de entrada de tramas
t_buffer_entr buffer_entr [NUM_MAX_TRAMAS];
int ins_buffer_entr = 0,   // Indica la posicin donde se debe insertar el
						   // siguiente paquete.
	sac_buffer_entr = 0;   // Indica la posicin de donde se debe sacar el
						   // siguiente paquete.

char dir_origen[TAM_DIR];  // Direccin fisica origen
char *tipo;            // tipo para la trama
// direccion fisica para broadcast
char BROAD_CAST[TAM_DIR] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};


/*************************************************************************
FUNCIONALIDAD: Obtiene las caracteristicas funcionales de la tarjeta de red.
PARAMETROS:
    registro: registro donde tenemos el valor asociado a
				cada cadena de caracteres
DEVUELVE: cadena con la funcionalidad de la tarjeta
/*************************************************************************/
char *devuelve(int registro){

  switch(registro){
	 case 1: return(" Funciones Bsicas presentes\n");
	 case 2: return(" Funciones Bsicas y extendidas\n");
	 case 5: return(" Funciones Bsicas y Altas prestaciones\n");
	 case 6: return(" Funciones Bsicas  Altas prestaciones y extendidas\n");
	 case 255: return(" Funciones No instaladas\n");

	 default: return(" ");
 }
}


/*************************************************************************
FUNCIONALIDAD: Imprime el error que ha ocurrido y aborta la ejecucion.
PARAMETROS:
    reg: registro con el numero del error (campo DH)
/*************************************************************************/
void error(struct REGPACK *reg) {

  printf("\nERROR, code error %d \n", HI(reg->r_dx));
  exit(1);
}


/*************************************************************************
FUNCIONALIDAD: Obtiene informacin de la tarjeta de red (version, clase,
			   tipo, numero, nombre y funcionalidad) y la imprime.
PARAMETROS:
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: Devuelve -1 si ha ocurrido algun error y 0 en caso contrario.
/*************************************************************************/
int driver_info(struct REGPACK *reg){

  reg->r_ax = 0x01 << 8;
  reg->r_ax = reg->r_ax | 0xFF;
  intr(0x60, reg);               // llamamos a la interrupcin 0x60


  if (reg->r_flags & CF)
	 return (-1);
  else {
	 printf("Versin: %d\n", reg->r_bx);
	 printf("Clase: %d\n", HI(reg->r_cx));
	 printf("Tipo: %d\n", reg->r_dx);
	 printf("Nmero: %d\n", LO(reg->r_cx));
	 printf("Nombre: %s\n", MK_FP(reg->r_ds, reg->r_si));
	 printf("Funcionalidad : %s\n", devuelve(LO(reg->r_ax)));
  }
  return 0;
}


/*************************************************************************
FUNCIONALIDAD: Obtiene la direccin fsica de la tarjeta de red.
PARAMETROS:
    handle: interfaz asociada con la tarjeta de red
    buffer: direccin donde se copiar la direccin fisica de la tarjeta
    len: longitud en bytes del buffer == tamao en bytes de la direccion
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: Devuelve -1 si ha ocurrido algun error y 0 en caso contrario.
/*************************************************************************/
int get_address(int handle, char *buffer, int len, struct REGPACK *reg){

  reg->r_ax = 0x06 << 8;
  reg->r_bx = handle;
  reg->r_es = FP_SEG((char far *)buffer);
  reg->r_di = FP_OFF((char far *)buffer);
  reg->r_cx = len;

  intr(0x60, reg);           // llamamos a la interrupcin 0x60

  if (reg->r_flags & CF)
	 return (-1);
  return 0;
}


/*************************************************************************
FUNCIONALIDAD: Enva un paquete almacenado en buffer.
PARAMETROS:
    buffer: Contiene la trama que describe completamente al paquete
			 (direcciones fisicas y tipo, datagrama y datos).
    len: tamao en bytes de la trama a enviar (contenido de buffer).
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: Devuelve -1 si ha ocurrido algun error y 0 en caso contrario.
/*************************************************************************/
int send_pkt(char *buffer, int len, struct REGPACK *reg) {

  reg->r_ax = 0x04 << 8;       /* Cargamos 4 en AH */
  reg->r_ds = FP_SEG((char far *)buffer);
  reg->r_si = FP_OFF((char far *)buffer);
  reg->r_cx = len;

  intr(0x60, reg);           // llamamos a la interrupcin 0x60/

  if (reg->r_flags & CF)
	 return (-1);

  return 0;
}


/*************************************************************************
FUNCIONALIDAD: Crea un paquete a partir de los datos pasados por parametro
			   listo para ser enviado.
PARAMETROS:
    dir_dest: direccin fisica destino
    dir_orig: direccin fisica origen
    tipo: tipo asociado con la tarjeta Ethernet
    datos: datos a enviar (datagrama + informacion)
    tamanio: tamao de los datos
DEVUELVE: Un paquete listo para ser enviado  NULL en caso de que no haya
		  suficiente memoria para crearlo.
/*************************************************************************/
char *make_pkt (char dir_dest[], char dir_orig[], char tipo[],
						 char *datos, int *tamanio) {

  char *paq,          // Paquete final a enviar
	   *ptr;          // Puntero auxiliar
  int tamanio_total;  // Tamao final del paquete

  if ((*tamanio + TAM_DIRS_TIPO) < TAM_MIN_TRAMA)
	 tamanio_total = TAM_MIN_TRAMA;
  else
	 tamanio_total = *tamanio + TAM_DIRS_TIPO;

  paq = (char *) (malloc(tamanio_total));
  if (paq != NULL) {
	 ptr = paq;
	 memcpy(ptr,dir_dest, TAM_DIR);
	 ptr += TAM_DIR;
	 memcpy(ptr,dir_orig, TAM_DIR);
	 ptr += TAM_DIR;
	 memcpy(ptr,tipo, TAM_TIPO);
	 ptr += TAM_TIPO;
	 memcpy(ptr,datos,*tamanio);

	 *tamanio = tamanio_total;
  }
  return (paq);
}


/*************************************************************************
FUNCIONALIDAD: Esta funcin fija el modo de recepcin.
	modo		Significado
	------------------------------------------
	1			aislado: no aceptamos ningn paquete.
	2			ok: aceptamos slo aquellos paquetes cuya direccin destino coinicida
				con la de la mquina.
	3			broadcast: modo 2 + paquetes con destino = FFFFFFFFFFFF
	6			fisgn: aceptamos todos los paquetes.
PARAMETROS:
    handle: interfaz asociada con la tarjeta de red
    mode: modo de transmisin.
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: Devuelve -1 si ha ocurrido algun error y 0 en caso contrario.
/*************************************************************************/
int set_rcv_mode(int handle, int mode, struct REGPACK *reg) {

	reg->r_ax =0x04 << 8;
	reg->r_bx = handle;
	reg->r_cx = mode;
	intr(0x60, reg);
	if (reg->r_flags & CF)
	  return (-1);

	return 0;
}


/*************************************************************************
FUNCIONALIDAD: Funcion asociada a la interrupcion que se activa cada vez
			   que llega en paquete (dos veces por paquete).
			   La primera interrupcin (AX=0), significa que ha llegado un
			   paquete y debemos devolverle un buffer para que la tarjeta
			   deposite el contenido del paquete recibido. Si le devolvemos
			   la direccin 0:0, significa que no hemos conseguido ningun
			   buffer y la segunda llamada no se realizara.
			   La segunda llamada significa que la tarjeta ya ha volcado
			   el contenido del paquete recibido en el buffer.
PARAMETROS:
    bp, di, si, ds, es, dx, cx, bc, ax, ip, cs, flags: registros de la maq.
/*************************************************************************/
void interrupt receiver(int bp, int di, int si, int ds, int es, int dx, int cx,
						int bx, int ax, int ip, int cs, int flags) {

  if (ax == 0) {
	 if (buffer_entr[ins_buffer_entr].libre) {
		es = FP_SEG((char far *)(buffer_entr[ins_buffer_entr].trama));
		di = FP_OFF((char far *)(buffer_entr[ins_buffer_entr].trama));
		buffer_entr[ins_buffer_entr].libre = 0;
		ins_buffer_entr = (ins_buffer_entr + 1) % NUM_MAX_TRAMAS;
	 } else {
		es = 0;
		di = 0;
	 }
  }
  // Con AX = 1 no se efectua ninguna operacin ya que los datos
  // estan almacenados en buffer_entr y podran ser accedidos
  // posteriormente.

}


/*************************************************************************
FUNCIONALIDAD: Crea la interfaz con la tarjeta de red estableciendo un
			   handle para podernos comunicar.
PARAMETROS:
    if_class: clase de la tarjeta de red
    if_type: tipo de tarjeta de red
    if_number: numero de la tarjeta
    type: tipo asociado para la interfaz con la tarjeta
    typelen: tamao en bytes del tipo
    receiver: funcin asociada a la recepcin de un paquete
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: haldle para comunicarnos con la tarjeta de red.
/*************************************************************************/
int access_type(int if_class, int if_type, int if_number, char *type,
				int typelen, void *receiver, struct REGPACK *reg) {

  reg->r_ax = 0x02 << 8;
  reg->r_ax = (if_class & 0xFF) | reg->r_ax;
  reg->r_bx = if_type;
  reg->r_dx = if_number & 0xFF;
  reg->r_cx = typelen;

  reg->r_ds = FP_SEG((char far *)type);
  reg->r_si = FP_OFF((char far *)type);

  reg->r_es = FP_SEG((char far *)receiver);
  reg->r_di = FP_OFF((char far *)receiver);

  intr(0x60, reg);          // llamamos a la interrupcin 0x60
  if (reg->r_flags & CF)
	  return (-1);

  return (reg->r_ax);
}


/*************************************************************************
FUNCIONALIDAD: Libera el tipo definido mediante el handle (deshabilita la
			   interfaz con la tarjeta ethernet).
PARAMETROS:
    handle: interfaz asociada con la tarjeta de red
    reg: registro donde devolver un error en caso de que exista (DH).
DEVUELVE: Devuelve -1 si ha ocurrido algun error y 0 en caso contrario.
/*************************************************************************/
int release(int handle, struct REGPACK *reg) {

	reg->r_ax = 0x03 << 8;		/* AH = 3 */
	reg->r_bx = handle;
	intr(0x60, reg);
	if (reg->r_flags & CF)
	  return (-1);
	else
	  return 0;
}

