/*************************************************************************
/ TRANSPOR.C
/ Implementa el nivel de transporte: interfaz entre el nivel de red y la
/ aplicacin final.
/*************************************************************************/
#include <io.h>
#include <dos.h>
#include "transpor.h"


   char datos [TAM_MAX_INFO];       // Se utiliza como buffer para leer los
									// datos de fichero.
   t_ventana ventana_salida; // ventana con los nuevos paquetes a enviar

/*************************************************************************
FUNCIONALIDAD: Genera la siguiente ventana a enviar/recibir segn los
			   paquetes que ya han sido enviados/recibidos con xito y el
			   nmero de paquetes totales que deben enviarse/recibirse.
PARAMETROS:
    ventana: ventana que contiene los paquetes recibidos de un conjunto
			  de TAM_VENTANA paquetes (ventana.vector), siendo el primero
			  de ellos el indicado por la variable 'ventana.primero'.
			  (ver fichero transpor.h)
    paq_totales: paquetes totales que se han de enviar/recibir
DEVUELVE: Una nueva ventana con los nuevos paquetes que se han de
		  enviar/recibir en posteriores transmisiones.
/*************************************************************************/
t_ventana genera_ventana_nueva(t_ventana ventana, int paq_totales){

   int i, // contador para los bucles
   primer_fallo = TAM_VENTANA; // primer paquete que no se ha logrado recibir

	for (i = 0;  i < TAM_VENTANA; i++){
		if (ventana.vector[i] == 1){    // ha encontrado el primero que fallo
			if (primer_fallo == TAM_VENTANA){
				 primer_fallo = i;
				 ventana_salida.primero =  ventana.primero + primer_fallo;
			}
			// copiamos los valores a partir del primero que ha fallado
			ventana_salida.vector[i - primer_fallo] = ventana.vector[i];
		}
	} // for

	// todos los paquetes se enviaron correctamanete
	if (primer_fallo == TAM_VENTANA)
	   ventana_salida.primero = ventana.primero + TAM_VENTANA;

	// Avanzamos la ventana deslizante aprovechando su tamao
	for (i = TAM_VENTANA - primer_fallo; i < TAM_VENTANA; i++)
		ventana_salida.vector[i] = 1;

  // Al mover la ventana tratamos ms paquetes de los que en realidad existen
  // por lo tanto, esos paquetes no los enviamos
	for (i =  0; i < TAM_VENTANA; i++)
	   if (ventana_salida.primero + i >= paq_totales)
		  ventana_salida.vector[i] = 0;

   // Aumentamos el numero de la ventana
   ventana_salida.num_ventana = ventana.num_ventana + 1;
   return ventana_salida;
}


/*************************************************************************
FUNCIONALIDAD:  Funcin utilizada para enviar los paquetes	que vienen
				indicados en el parmetro ventana, siendo estos	los
				paquetes ubicados en el fichero handle.
				Destacar que una ventana que no pide ningn paquete significa
				que se ha compretado la trasferencia de todos los paquetes.
PARAMETROS:
    ventana: contiene los paquetes que han de ser enviados.
    handle: descriptor del fichero donde residen los datos a enviar
    n_logico_og: nombre lgico de la mquina origen
    n_logico_dt: nombre lgico de la mquina destino
    flags: flags de los paquetes a enviar
    num_usr: numero del usuario.
DEVUELVE:
     0, se ha enviado todo el fichero ya que no se tuvo que ahora no se
	  tuvo que manda ningn paquete.
     1, se han enviado paquetes
/**************************************************************************/
int enviar_paquetes_de_ventana (t_ventana ventana, int handle,
								char n_logico_og[],	char n_logico_dt[],
								int flags, int num_usr){

   int i;                           // variable auxiliar
   t_datagrama datagrama;           // Datagrama para enviar los paquetes
   int envio_finalizado = 1;        // Indica si se ha enviado todo el fichero

  // Crea el datagrama correspondiente al envio de los paquetes indicados
  // en el parametro ventana, obtenidos del fichero 'handle'.
  strcpy(datagrama.logico_og, n_logico_og);
  strcpy(datagrama.logico_dt, n_logico_dt);
  datagrama.aux1 = num_usr;
  datagrama.pq_total = ceil (filelength (handle) / (float)(TAM_MAX_INFO));
  datagrama.flags = flags;

  // Envia todos los paquetes que la ventana
  for (i = 0; i < TAM_VENTANA; i++)
	 // Si hay que enviar el paquete X
	 if (ventana.vector [i] == 1) {
		// Lee del fichero y actualiza las variables del datagrama
		envio_finalizado = 0;
		lseek (handle, (ventana.primero + i) * TAM_MAX_INFO, SEEK_SET);
		datagrama.tam_dat = read (handle, datos, TAM_MAX_INFO);
		datagrama.num_pq = ventana.primero + i;
		// Envia el paquete
		enviar_paq (datagrama, datos);
	 }

  if (envio_finalizado)
	 return (0);
  else
	 return (1);
}

